import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { RequestItem } from '../../query/request-item';
import { DatasetProfileDefinitionModel } from '../../model/dataset-profile-definition/dataset-profile-definition';
import { DatasetProfileModel } from '../../model/dataset/dataset-profile';
import { DatasetWizardModel } from '../../model/dataset/dataset-wizard';
import { DmpModel } from '../../model/dmp/dmp';
import { DatasetProfileCriteria } from '../../query/dataset-profile/dataset-profile-criteria';
import { DmpCriteria } from '../../query/dmp/dmp-criteria';
import { BaseHttpService } from '../http/base-http.service';

@Injectable()
export class DatasetWizardService {

	private actionUrl: string;
	private headers = new HttpHeaders();

	constructor(private http: BaseHttpService, private httpClient: HttpClient) {
		this.actionUrl = environment.Server + 'datasetwizard/';
	}

	// public userDmps(criteria: RequestItem<DmpCriteria>): Observable<DmpModel[]> {
	// 	return this.http.post<DmpModel[]>(this.actionUrl + 'userDmps', criteria, { headers: this.headers });
	// }

	public getAvailableProfiles(criteria: RequestItem<DatasetProfileCriteria>): Observable<DatasetProfileModel[]> {
		return this.http.post<DatasetProfileModel[]>(this.actionUrl + 'getAvailableProfiles/', criteria, { headers: this.headers });
	}

	public getSingle(id: String): Observable<DatasetWizardModel> {
		return this.http.get<DatasetWizardModel>(this.actionUrl + id, { headers: this.headers });  // + 'getSingle/'
	}

	public delete(id: string): Observable<DatasetWizardModel> {
		return this.http.delete<DatasetWizardModel>(this.actionUrl + id, { headers: this.headers });
	}

	createDataset(datasetModel: DatasetWizardModel): Observable<DatasetWizardModel> {
		return this.http.post<DatasetWizardModel>(this.actionUrl, datasetModel, { headers: this.headers });
	}

	public downloadPDF(id: string): Observable<HttpResponse<Blob>> {
		return this.httpClient.get(this.actionUrl + 'getPDF/' + id, { responseType: 'blob', observe: 'response', headers: this.headers });
	}

	public downloadXML(id: string): Observable<HttpResponse<Blob>> {
		let headerXml: HttpHeaders = this.headers.set('Content-Type', 'application/xml')
		return this.httpClient.get(this.actionUrl + id, { responseType: 'blob', observe: 'response', headers: headerXml }); // + 'getXml/'
	}

	public getDefinition(id: String): Observable<DatasetProfileDefinitionModel> {
		return this.http.get<DatasetProfileDefinitionModel>(this.actionUrl + 'get/' + id, { headers: this.headers });
	}

	unlock(id: String): Observable<DatasetWizardModel> {
		return this.http.get<DatasetWizardModel>(this.actionUrl + id + '/unlock', { headers: this.headers });
	}
}
