import { HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from '../../../../environments/environment';
import { DataTableData } from '../../model/data-table/data-table-data';
import { DataTableRequest } from '../../model/data-table/data-table-request';
import { ProjectListingModel } from '../../model/project/project-listing';
import { ProjectCriteria } from '../../query/project/project-criteria';
import { RequestItem } from '../../query/request-item';
import { BaseHttpService } from '../http/base-http.service';


@Injectable()
export class ProjectService {

	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(private http: BaseHttpService) {
		this.actionUrl = environment.Server + 'projects/';
	}

	getPaged(dataTableRequest: DataTableRequest<ProjectCriteria>, fieldsGroup?: string): Observable<DataTableData<ProjectListingModel>> {
		if (fieldsGroup) {
			return this.http.post<DataTableData<ProjectListingModel>>(this.actionUrl + 'paged?fieldsGroup=' + fieldsGroup, dataTableRequest, { headers: this.headers });
		}
		else {
			return this.http.post<DataTableData<ProjectListingModel>>(this.actionUrl + 'paged?fieldsGroup=' + 'autocomplete', dataTableRequest, { headers: this.headers });
		}
	}

	getWithExternal(requestItem: RequestItem<ProjectCriteria>): Observable<ProjectListingModel[]> {
		return this.http.post<ProjectListingModel[]>(this.actionUrl + 'external', requestItem, { headers: this.headers });
	}

	getSingle(id: string): Observable<ProjectListingModel> {
		return this.http.get<ProjectListingModel>(this.actionUrl + id, { headers: this.headers });
	}

	createProject(projectModel: ProjectListingModel): Observable<ProjectListingModel> {
		return this.http.post<ProjectListingModel>(this.actionUrl, projectModel, { headers: this.headers });
	}

	// Actually sets it inactive.
	delete(id: String): Observable<ProjectListingModel> {
		return this.http.delete<ProjectListingModel>(this.actionUrl + id, { headers: this.headers });
	}

}
