import { FormGroup } from '@angular/forms';
import { DatasetProfileComboBoxType } from '../../../../../core/common/enum/dataset-profile-combo-box-type';
import { WordListFieldData } from '../../../../../core/model/dataset-profile-definition/field-data/field-data';
import { FieldDataEditorModel } from './field-data-editor-model';
import { FieldDataOptionEditorModel } from './field-data-option-editor-model';

export class WordListFieldDataEditorModel extends FieldDataEditorModel<WordListFieldDataEditorModel> {
	public type: DatasetProfileComboBoxType = DatasetProfileComboBoxType.WordList;
	public options: Array<FieldDataOptionEditorModel>;

	buildForm(): FormGroup {
		const formGroup = this.formBuilder.group({
			type: [this.type],
			label: [this.label]
		});
		const optionsFormArray = new Array<FormGroup>();
		if (this.options) {
			this.options.forEach(item => {
				const form: FormGroup = item.buildForm();
				optionsFormArray.push(form);
			});
		}
		formGroup.addControl('options', this.formBuilder.array(optionsFormArray));

		return formGroup;
	}

	fromModel(item: WordListFieldData): WordListFieldDataEditorModel {
		this.type = item.type;
		if (item.options) { this.options = item.options.map(x => new FieldDataOptionEditorModel().fromModel(x)); }
		this.label = item.label;
		return this;
	}
}
