﻿
import { FormGroup, Validators } from '@angular/forms';
import { FieldSet } from '../../../../core/model/admin/dataset-profile/dataset-profile';
import { BaseFormModel } from '../../../../core/model/base-form-model';
import { FieldEditorModel } from './field-editor-model';
import { MultiplicityEditorModel } from './multiplicity-editor-model';

export class FieldSetEditorModel extends BaseFormModel {

	public fields: Array<FieldEditorModel> = new Array<FieldEditorModel>();
	public id: string;
	public ordinal: number;
	public multiplicity: MultiplicityEditorModel = new MultiplicityEditorModel();
	public title: string;
	public description: string;
	public extendedDescription: string;
	public hasCommentField: boolean;

	fromModel(item: FieldSet): FieldSetEditorModel {

		if (item.fields) { this.fields = item.fields.map(x => new FieldEditorModel().fromModel(x)); }
		if (item.multiplicity) this.multiplicity = new MultiplicityEditorModel().fromModel(item.multiplicity);
		this.id = item.id;
		this.ordinal = item.ordinal;
		this.title = item.title;
		this.description = item.description;
		this.extendedDescription = item.extendedDescription;
		this.hasCommentField = item.hasCommentField;
		return this;
	}

	buildForm(): FormGroup {
		const formGroup = this.formBuilder.group({
			id:[this.id,[Validators.required,Validators.pattern('^[^<_>]+$')]],
			ordinal: [this.ordinal],
			title: [this.title],
			description: [this.description],
			extendedDescription: [this.extendedDescription],
			hasCommentField: [this.hasCommentField]
		});
		const fieldsFormArray = new Array<FormGroup>();
		this.fields.forEach(item => {
			const form: FormGroup = item.buildForm();
			fieldsFormArray.push(form);
		});
		formGroup.addControl('fields', this.formBuilder.array(fieldsFormArray));
		formGroup.addControl('multiplicity', this.multiplicity.buildForm());

		return formGroup;
	}
}
