import { DataSource } from '@angular/cdk/table';
import { Component, OnInit, ViewChild } from '@angular/core';
import { MatPaginator, MatSort, PageEvent } from '@angular/material';
import { ActivatedRoute, Params, Router } from '@angular/router';
import { Observable } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { BaseComponent } from '../../../../core/common/base/base.component';
import { DataTableRequest } from '../../../../core/model/data-table/data-table-request';
import { DatasetListingModel } from '../../../../core/model/dataset/dataset-listing';
import { DmpModel } from '../../../../core/model/dmp/dmp';
import { DatasetProfileCriteria } from '../../../../core/query/dataset-profile/dataset-profile-criteria';
import { DatasetProfileService } from '../../../../core/services/dataset-profile/dataset-profile.service';
import { DmpService } from '../../../../core/services/dmp/dmp.service';
import { DatasetProfileCriteriaComponent } from './criteria/dataset-profile.component';

@Component({
	selector: 'app-dataset-profile-listing-component',
	templateUrl: 'dataset-profile-listing.component.html',
	styleUrls: ['./dataset-profile-listing.component.scss']
})
export class DatasetProfileListingComponent extends BaseComponent implements OnInit {

	@ViewChild(MatPaginator) _paginator: MatPaginator;
	@ViewChild(MatSort) sort: MatSort;
	@ViewChild(DatasetProfileCriteriaComponent) criteria: DatasetProfileCriteriaComponent;


	dataSource: DatasetDataSource | null;
	displayedColumns: String[] = ['label', 'description', 'created', 'actions'];
	pageEvent: PageEvent;
	titlePrefix: String;
	dmpId: String;

	constructor(
		private datasetService: DatasetProfileService,
		private router: Router,
		private route: ActivatedRoute,
		private dmpService: DmpService
	) {
		super();
	}


	ngOnInit() {
		this.route.params
			.pipe(takeUntil(this._destroyed))
			.subscribe((params: Params) => {
				this.dmpId = params['dmpId'];
				if (this.dmpId != null) { this.setDmpTitle(this.dmpId); }
				this.criteria.setCriteria(this.getDefaultCriteria(this.dmpId));
				this.refresh();
				this.criteria.setRefreshCallback(() => this.refresh());
			});
	}

	setDmpTitle(dmpId: String) {
		this.dmpService.getSingle(dmpId).map(data => data as DmpModel)
			.pipe(takeUntil(this._destroyed))
			.subscribe(data => {
				this.titlePrefix = data.label;
			});
	}

	refresh() {
		this.dataSource = new DatasetDataSource(this.datasetService, this._paginator, this.sort, this.criteria);
	}

	rowClick(rowId: String) {
		this.router.navigate(['dataset-profiles/' + rowId]);
	}

	getDefaultCriteria(dmpId: String): DatasetProfileCriteria {
		const defaultCriteria = new DatasetProfileCriteria();
		return defaultCriteria;
	}

	clone(id: string) {
		this.router.navigate(['dataset-profiles/clone/' + id]);
	}

	// makeItPublic(id: String) {
	//   debugger;
	//   this.datasetService.makeDatasetPublic(id).pipe(takeUntil(this._destroyed)).subscribe();
	// }
}

export class DatasetDataSource extends DataSource<DatasetListingModel> {

	totalCount = 0;

	constructor(
		private _service: DatasetProfileService,
		private _paginator: MatPaginator,
		private _sort: MatSort,
		private _criteria: DatasetProfileCriteriaComponent
	) {
		super();

	}

	connect(): Observable<DatasetListingModel[]> {
		const displayDataChanges = [
			this._paginator.page
			//this._sort.matSortChange
		];

		return Observable.merge(...displayDataChanges)
			.startWith(null)
			.switchMap(() => {
				const startIndex = this._paginator.pageIndex * this._paginator.pageSize;
				let fields: Array<string> = new Array();
				if (this._sort.active) { fields = this._sort.direction === 'asc' ? ['+' + this._sort.active] : ['-' + this._sort.active]; }
				const request = new DataTableRequest<DatasetProfileCriteria>(startIndex, this._paginator.pageSize, { fields: fields });
				request.criteria = this._criteria.criteria;
				return this._service.getPaged(request);
			})
			/*.catch((error: any) => {
			  this._snackBar.openFromComponent(SnackBarNotificationComponent, {
				data: { message: 'GENERAL.SNACK-BAR.FORMS-BAD-REQUEST', language: this._languageService },
				duration: 3000,
				extraClasses: ['snackbar-warning']
			  });
			  //this._criteria.criteria.onCallbackError(error);
			  return Observable.of(null);
			})*/
			.map(result => {
				return result;
			})
			.map(result => {
				if (!result) { return []; }
				if (this._paginator.pageIndex === 0) { this.totalCount = result.totalCount; }
				return result.data;
			});
	}

	disconnect() {
		// No-op
	}
}
