import { Component, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { ValidationErrorModel } from '../../../../common/forms/validation/error-model/validation-error-model';
import { DatasetStatus } from '../../../../core/common/enum/dataset-status';
import { ExternalSourceItemModel } from '../../../../core/model/external-sources/external-source-item';
import { DatasetCriteria } from '../../../../core/query/dataset/dataset-criteria';
import { TagCriteria } from '../../../../core/query/tag/tag-criteria';
import { ExternalSourcesService } from '../../../../core/services/external-sources/external-sources.service';
import { EnumUtils } from '../../../../core/services/utilities/enum-utils.service';
import { RequestItem } from '../../../../core/query/request-item';
import { BaseCriteriaComponent } from '../../../misc/criteria/base-criteria.component';

@Component({
	selector: 'app-dataset-criteria-component',
	templateUrl: './dataset-criteria.component.html',
	styleUrls: ['./dataset-criteria.component.scss']
})
export class DatasetCriteriaComponent extends BaseCriteriaComponent implements OnInit {

	public criteria: any;
	public filteringTagsAsync = false;
	public filteredTags: ExternalSourceItemModel[];

	statuses = DatasetStatus;

	tagsAutoCompleteConfiguration = {
		filterFn: this.filterTags.bind(this),
		initialItems: (excludedItems: any[]) => this.filterTags('').map(result => result.filter(resultItem => excludedItems.map(x => x.id).indexOf(resultItem.id) === -1)),
		displayFn: (item) => item['name'],
		titleFn: (item) => item['name']
	};

	constructor(
		private externalSourcesService: ExternalSourcesService,
		public enumUtils: EnumUtils
	) {
		super(new ValidationErrorModel());
	}

	ngOnInit() {
		super.ngOnInit();
		if (this.criteria == null) { this.criteria = {}; }
	}

	setCriteria(criteria: DatasetCriteria): void {
		this.criteria = criteria;
	}

	onCallbackError(error: any) {
		this.setErrorModel(error.error);
	}

	controlModified(): void {
		this.clearErrorModel();
		if (this.refreshCallback != null &&
			(this.criteria.like == null || this.criteria.like.length === 0 || this.criteria.like.length > 2)
		) {
			this.refreshCallback();
		}
	}

	filterTags(value: string): Observable<ExternalSourceItemModel[]> {

		this.filteredTags = undefined;
		this.filteringTagsAsync = true;

		const requestItem: RequestItem<TagCriteria> = new RequestItem();
		const criteria: TagCriteria = new TagCriteria();
		criteria.like = value;
		requestItem.criteria = criteria;
		return this.externalSourcesService.searchDatasetTags(requestItem);

	}
}
