import { Component, Input, OnInit } from '@angular/core';
import { FormBuilder, FormControl } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { takeUntil } from 'rxjs/operators';
import { ValidationErrorModel } from '../../../../common/forms/validation/error-model/validation-error-model';
import { ProjectListingModel } from '../../../../core/model/project/project-listing';
import { DmpCriteria } from '../../../../core/query/dmp/dmp-criteria';
import { ProjectCriteria } from '../../../../core/query/project/project-criteria';
import { ProjectService } from '../../../../core/services/project/project.service';
import { MultipleAutoCompleteConfiguration } from '../../../../library/auto-complete/multiple/multiple-auto-complete-configuration';
import { RequestItem } from '../../../../core/query/request-item';
import { BaseCriteriaComponent } from '../../../misc/criteria/base-criteria.component';
import { DataTableRequest } from '../../../../core/model/data-table/data-table-request';

@Component({
	selector: 'app-dmp-criteria-component',
	templateUrl: './dmp-criteria.component.html',
	styleUrls: ['./dmp-criteria.component.scss'],
})
export class DmpCriteriaComponent extends BaseCriteriaComponent implements OnInit {

	@Input() showProject: boolean;
	filteringProjectsAsync = false;
	filteredProjects: ProjectListingModel[];
	public formGroup = new FormBuilder().group({
		like: new FormControl(),
		projects: new FormControl()
	});
	projectAutoCompleteConfiguration: MultipleAutoCompleteConfiguration = {
		filterFn: this.filterProject.bind(this),
		initialItems: (excludedItems: any[]) => this.filterProject('').map(result => result.filter(resultItem => excludedItems.map(x => x.id).indexOf(resultItem.id) === -1)),
		displayFn: (item) => item['label'],
		titleFn: (item) => item['label']
	};

	constructor(
		public language: TranslateService,
		public projectService: ProjectService,
		public formBuilder: FormBuilder
	) {
		super(new ValidationErrorModel());
	}

	ngOnInit() {
		super.ngOnInit();

		this.formGroup.get('projects').valueChanges
			.pipe(takeUntil(this._destroyed))
			.subscribe(x => this.controlModified());
		this.formGroup.get('like').valueChanges
			.pipe(takeUntil(this._destroyed))
			.subscribe(x => this.controlModified());
		//if (this.criteria == null) { this.criteria = new DataManagementPlanCriteria(); }
	}

	setCriteria(criteria: DmpCriteria): void {
		this.formGroup.get('like').patchValue(criteria.like);
		this.formGroup.get('projects').patchValue(criteria.projects);
	}

	onCallbackError(error: any) {
		this.setErrorModel(error.error);
	}

	controlModified(): void {
		this.clearErrorModel();
		if (this.refreshCallback != null &&
			(this.formGroup.get('like').value == null || this.formGroup.get('like').value.length === 0 || this.formGroup.get('like').value.length > 2)
		) {
			this.refreshCallback();
		}
	}

	filterProject(query: string) {
		const fields: Array<string> = new Array<string>();
		fields.push('asc');
		const projectRequestItem: DataTableRequest<ProjectCriteria> = new DataTableRequest(0, null, { fields: fields });
		projectRequestItem.criteria = new ProjectCriteria();
		projectRequestItem.criteria.like = query;
		return this.projectService.getPaged(projectRequestItem, "autocomplete").map(x => x.data);
	}
}
